@@ -27,7 +27,6 @@
 
 #include <QDebug>
 
-#include <kdebug.h>
 #include <QStringList>
 
 // If DUMPTREE is defined, ModelInsertCommand dumps the tree of what it is inserting.
@@ -52,6 +51,13 @@ QModelIndex DynamicTreeModel::index(int row, int column, const QModelIndex &pare
 
   QList<QList<qint64> > childIdColumns = m_childItems.value(parent.internalId());
 
+  const qint64 grandParent = findParentId(parent.internalId());
+  if (grandParent >= 0) {
+    QList<QList<qint64> > parentTable = m_childItems.value(grandParent);
+    Q_ASSERT(parent.column() < parentTable.size());
+    QList<qint64> parentSiblings = parentTable.at(parent.column());
+    Q_ASSERT(parent.row() < parentSiblings.size());
+  }
 
   if (childIdColumns.size() == 0)
     return QModelIndex();
@@ -189,7 +195,7 @@ bool DynamicTreeModel::dropMimeData(const QMimeData* data, Qt::DropAction action
   {
     id = ba.toInt(&ok);
     if (!ok)
-      kDebug() << ba;
+      qDebug() << ba;
     Q_ASSERT(ok);
 
     _row = -1;
@@ -620,9 +626,9 @@ void ModelInsertAndRemoveQueuedCommand::doCommand()
 {
   QModelIndex parent = findIndex(m_rowNumbers);
 
-  connect (this, SIGNAL(beginInsertRows(const QModelIndex &, int, int)), SLOT(queuedBeginInsertRows(const QModelIndex &, int, int)), Qt::QueuedConnection);
+  connect (this, SIGNAL(beginInsertRows(QModelIndex,int,int)), SLOT(queuedBeginInsertRows(QModelIndex,int,int)), Qt::QueuedConnection);
   connect (this, SIGNAL(endInsertRows()), SLOT(queuedEndInsertRows()), Qt::QueuedConnection);
-  connect (this, SIGNAL(beginRemoveRows(const QModelIndex &, int, int)), SLOT(queuedBeginRemoveRows(const QModelIndex &, int, int)), Qt::QueuedConnection);
+  connect (this, SIGNAL(beginRemoveRows(QModelIndex,int,int)), SLOT(queuedBeginRemoveRows(QModelIndex,int,int)), Qt::QueuedConnection);
   connect (this, SIGNAL(endRemoveRows()), SLOT(queuedEndRemoveRows()), Qt::QueuedConnection);
 
   emit beginInsertRows(parent, m_startRow, m_endRow);
@@ -945,6 +951,60 @@ void ModelResetCommand::doCommand()
   m_model->endResetModel();
 }
 
+ModelLayoutChangeCommand::ModelLayoutChangeCommand(DynamicTreeModel* model, QObject* parent)
+  : ModelChangeCommand(model, parent)
+{
+
+}
+
+ModelLayoutChangeCommand::~ModelLayoutChangeCommand()
+{
+
+}
+
+void ModelLayoutChangeCommand::setInitialTree(const QString& treeString)
+{
+  m_treeString = treeString;
+}
+
+void ModelLayoutChangeCommand::setPersistentChanges(const QList< ModelLayoutChangeCommand::PersistentChange >& changes)
+{
+  m_changes = changes;
+}
+
+void ModelLayoutChangeCommand::doCommand()
+{
+  m_model->layoutAboutToBeChanged();
+  QModelIndexList oldList;
+
+  foreach(const PersistentChange &change, m_changes)
+  {
+    const IndexFinder oldFinder(m_model, change.oldPath);
+    oldList << oldFinder.getIndex();
+  }
+
+  bool blocked = m_model->blockSignals(true);
+  m_model->clear();
+  if (!m_treeString.isEmpty())
+  {
+    ModelInsertCommand *ins = new ModelInsertCommand(m_model);
+    ins->setStartRow(0);
+    ins->interpret(m_treeString);
+    ins->doCommand();
+  }
+
+  QModelIndexList newList;
+  foreach(const PersistentChange &change, m_changes)
+  {
+    const IndexFinder newFinder(m_model, change.newPath);
+    newList << newFinder.getIndex();
+  }
+  m_model->changePersistentIndexList(oldList, newList);
+
+  m_model->blockSignals(blocked);
+  m_model->layoutChanged();
+}
+
 
 
 
