@@ -37,8 +37,6 @@
 #include <xml/dom_stringimpl.h>
 #include <xml/dom_docimpl.h>
 
-#include <misc/loader.h>
-
 #include <rendering/font.h>
 #include <rendering/render_style.h>
 
@@ -199,7 +197,8 @@ DOMString CSSStyleDeclarationImpl::removeProperty(const DOMString &propertyName)
 
 DOMString CSSStyleDeclarationImpl::getPropertyValue( int propertyID ) const
 {
-    if(!m_lstValues) return DOMString();
+    if (!m_lstValues || m_lstValues->isEmpty()) return DOMString();
+
     CSSValueImpl* value = getPropertyCSSValue( propertyID );
     if ( value )
         return value->cssText();
@@ -429,7 +428,7 @@ DOMString CSSStyleDeclarationImpl::getShortHandValue( const int* properties, int
 
  CSSValueImpl *CSSStyleDeclarationImpl::getPropertyCSSValue( int propertyID ) const
 {
-    if(!m_lstValues) return 0;
+    if (!m_lstValues || m_lstValues->isEmpty()) return 0;
 
     QListIterator<CSSProperty*> lstValuesIt(*m_lstValues);
     CSSProperty *current;
@@ -486,8 +485,6 @@ static void initShorthandMap(QHash<int, PropertyLonghand>& shorthandMap)
     #define SET_SHORTHAND_MAP_ENTRY(map, propID, array) \
         map.insert(propID, PropertyLonghand(array, sizeof(array) / sizeof(array[0])))
 
-    // FIXME: The 'font' property has "shorthand nature" but is not parsed as a shorthand.
-
     // Do not change the order of the following four shorthands, and keep them together.
     static const int borderProperties[4][3] = {
         { CSS_PROP_BORDER_TOP_COLOR, CSS_PROP_BORDER_TOP_STYLE, CSS_PROP_BORDER_TOP_WIDTH },
@@ -623,36 +620,48 @@ static void initShorthandMap(QHash<int, PropertyLonghand>& shorthandMap)
         CSS_PROP__KHTML_BORDER_BOTTOM_RIGHT_RADIUS
     };
     SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP__KHTML_BORDER_RADIUS, prefixedBorderRadiusProperties);
-                                                
+
     static const int markerProperties[] = {
-        CSS_PROP_MARKER_START, 
+        CSS_PROP_MARKER_START,
         CSS_PROP_MARKER_MID,
         CSS_PROP_MARKER_END
     };
     SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_MARKER, markerProperties);
 
+    static const int fontProperties[] = {
+        CSS_PROP_FONT_STYLE,
+        CSS_PROP_FONT_VARIANT,
+        CSS_PROP_FONT_WEIGHT,
+        CSS_PROP_FONT_SIZE,
+        CSS_PROP_LINE_HEIGHT,
+        CSS_PROP_FONT_FAMILY
+    };
+    SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_FONT, fontProperties);
+
     #undef SET_SHORTHAND_MAP_ENTRY
 }
 
 // -------------------------------------------
 
-void CSSStyleDeclarationImpl::removeProperty(int propertyID,
-                                             DOM::DOMString* old)
+void CSSStyleDeclarationImpl::removeProperty(int propertyID, DOM::DOMString* old)
 {
-    if(!m_lstValues)
+    if (!m_lstValues || m_lstValues->isEmpty())
         return;
 
+    bool changed = false;
+
     static QHash<int, PropertyLonghand> shorthandMap;
     if (shorthandMap.isEmpty())
         initShorthandMap(shorthandMap);
 
     PropertyLonghand longhand = shorthandMap.value(propertyID);
     if (longhand.length()) {
-        removePropertiesInSet(longhand.properties(), longhand.length());
-        // FIXME: Return an equivalent shorthand when possible.
-        return;
+        // Remove shorthand's longhand subproperties
+        changed = removePropertiesInSet(longhand.properties(), longhand.length());
     }
 
+    // FIXME: Return an equivalent shorthand when possible.
+
     QMutableListIterator<CSSProperty*> lstValuesIt(*m_lstValues);
     CSSProperty *current;
     lstValuesIt.toBack();
@@ -663,13 +672,17 @@ void CSSStyleDeclarationImpl::removeProperty(int propertyID,
                 *old = current->value()->cssText();
             delete lstValuesIt.value();
             lstValuesIt.remove();
-            setChanged();
+            changed = true;
             break;
         }
      }
+
+     if (changed) {
+        setChanged();
+     }
 }
 
-void CSSStyleDeclarationImpl::removePropertiesInSet(const int* set, unsigned length)
+bool CSSStyleDeclarationImpl::removePropertiesInSet(const int* set, unsigned length)
 {
     bool changed = false;
     for (unsigned i = 0; i < length; i++) {
@@ -686,8 +699,8 @@ void CSSStyleDeclarationImpl::removePropertiesInSet(const int* set, unsigned len
             }
         }
     }
-    if (changed)
-        setChanged();
+
+    return changed;
 }
 
 void CSSStyleDeclarationImpl::setChanged()
@@ -719,7 +732,7 @@ void CSSStyleDeclarationImpl::clear()
 
 bool CSSStyleDeclarationImpl::getPropertyPriority( int propertyID ) const
 {
-    if ( m_lstValues) {
+    if (m_lstValues && !m_lstValues->isEmpty()) {
 	QListIterator<CSSProperty*> lstValuesIt(*m_lstValues);
 	CSSProperty *current;
 	while (lstValuesIt.hasNext()) {
@@ -821,24 +834,26 @@ CSSRuleImpl *CSSStyleDeclarationImpl::parentRule() const
 
 DOM::DOMString CSSStyleDeclarationImpl::cssText() const
 {
-    DOMString result;
+    if (!m_lstValues || m_lstValues->isEmpty()) {
+        return DOMString();
+    }
 
+    DOMString result;
     const CSSProperty* positionXProp = 0;
     const CSSProperty* positionYProp = 0;
 
-    if ( m_lstValues) {
-	QListIterator<CSSProperty*> lstValuesIt(*m_lstValues);
-	while (lstValuesIt.hasNext()) {
-	    const CSSProperty* cur = lstValuesIt.next();
-            if (cur->id() == CSS_PROP_BACKGROUND_POSITION_X)
-                positionXProp = cur;
-            else if (cur->id() == CSS_PROP_BACKGROUND_POSITION_Y)
-                positionYProp = cur;
-            else
-                result += cur->cssText();
-	}
+    QListIterator<CSSProperty*> lstValuesIt(*m_lstValues);
+    while (lstValuesIt.hasNext()) {
+        const CSSProperty* cur = lstValuesIt.next();
+        if (cur->id() == CSS_PROP_BACKGROUND_POSITION_X)
+            positionXProp = cur;
+        else if (cur->id() == CSS_PROP_BACKGROUND_POSITION_Y)
+            positionYProp = cur;
+        else
+            result += cur->cssText();
     }
 
+
     // FIXME: This is a not-so-nice way to turn x/y positions into single background-position in output.
     // It is required because background-position-x/y are non-standard properties and generated output
     // would not work in Firefox
@@ -947,14 +962,17 @@ void CSSValueListImpl::append(CSSValueImpl *val)
 
 DOM::DOMString CSSValueListImpl::cssText() const
 {
-    DOMString result = "";
+    DOMString separatorString;
+    if (m_separator == Comma) {
+        separatorString = DOMString(", ");
+    } else { // Space
+        separatorString = DOMString(" ");
+    }
 
+    DOMString result = "";
     for (QListIterator<CSSValueImpl*> iterator(m_values); iterator.hasNext();) {
         if (!result.isEmpty()) {
-            if (m_separator == Comma)
-                result += ", ";
-            else if (m_separator == Space)
-                result += " ";
+            result += separatorString;
         }
         result += iterator.next()->cssText();
     }
@@ -1050,12 +1068,12 @@ void CSSPrimitiveValueImpl::cleanup()
     m_type = 0;
 }
 
-int CSSPrimitiveValueImpl::computeLength( khtml::RenderStyle *style, int logicalDpiY)
+int CSSPrimitiveValueImpl::computeLength(khtml::RenderStyle *style, khtml::RenderStyle *rootStyle, int logicalDpiY)
 {
-    return snapValue( computeLengthFloat( style, logicalDpiY ) );
+    return snapValue( computeLengthFloat(style, rootStyle, logicalDpiY) );
 }
 
-double CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, int logicalDpiY)
+double CSSPrimitiveValueImpl::computeLengthFloat(khtml::RenderStyle *style, khtml::RenderStyle *rootStyle, int logicalDpiY)
 {
     unsigned short type = primitiveType();
 
@@ -1072,11 +1090,21 @@ double CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, int
         factor = style->font().pixelSize();
         break;
     case CSSPrimitiveValue::CSS_EXS:
-	{
-        QFontMetrics fm = style->fontMetrics();
-        factor = fm.xHeight();
+        factor = style->htmlFont().xHeight();
+        break;
+    case CSSPrimitiveValue::CSS_CHS: {
+        const int zw = style->htmlFont().zeroCharWidth();
+        if (zw != -1) {
+            factor = zw;
+        } else {
+            // assume 0.5em
+            return ((double)0.5 * style->font().pixelSize());
+        }
+        break;
+        }
+    case CSSPrimitiveValue::CSS_REMS:
+        factor = rootStyle->font().pixelSize();
         break;
-	}
     case CSSPrimitiveValue::CSS_PX:
         break;
     case CSSPrimitiveValue::CSS_CM:
@@ -1196,6 +1224,12 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 	case CSSPrimitiveValue::CSS_EXS:
 	    text = DOMString(QString::number( m_value.num ) + "ex");
 	    break;
+        case CSSPrimitiveValue::CSS_CHS:
+            text = DOMString(QString::number( m_value.num ) + "ch");
+            break;
+        case CSSPrimitiveValue::CSS_REMS:
+            text = DOMString(QString::number( m_value.num ) + "rem");
+            break;
 	case CSSPrimitiveValue::CSS_PX:
 	    text = DOMString(QString::number( m_value.num ) + "px");
 	    break;
@@ -1370,23 +1404,20 @@ void PairImpl::setSecond(CSSPrimitiveValueImpl* second)
 CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, StyleBaseImpl* style)
     : CSSPrimitiveValueImpl(url, CSSPrimitiveValue::CSS_URI)
 {
-    khtml::DocLoader *docLoader = 0;
-    const StyleBaseImpl *root = style;
-    while (root->parent())
-	root = root->parent();
-    if (root->isCSSStyleSheet())
-	docLoader = static_cast<const CSSStyleSheetImpl*>(root)->docLoader();
-    if (docLoader) {
-    KUrl fullURL( style->baseURL(), khtml::parseURL(url).string() );
-    m_image = docLoader->requestImage( fullURL.url() );
-    if(m_image) m_image->ref(this);
-}
+    m_image = 0;
+    const DOMString imgUrl = url.trimSpaces();
+    if (!imgUrl.isEmpty()) {
+        m_fullImageUrl = KUrl(style->baseURL(), imgUrl.string()).url();
+    } else {
+        m_fullImageUrl.clear();
+    }
 }
 
 CSSImageValueImpl::CSSImageValueImpl()
     : CSSPrimitiveValueImpl(CSS_VAL_NONE)
 {
     m_image = 0;
+    m_fullImageUrl.clear();
 }
 
 CSSImageValueImpl::~CSSImageValueImpl()
@@ -1394,6 +1425,17 @@ CSSImageValueImpl::~CSSImageValueImpl()
     if(m_image) m_image->deref(this);
 }
 
+khtml::CachedImage *CSSImageValueImpl::requestCssImage(DocumentImpl* doc)
+{
+    if (!m_image && !m_fullImageUrl.isEmpty()) {
+        m_image = doc->docLoader()->requestImage(m_fullImageUrl);
+        if (m_image) {
+            m_image->ref(this);
+        }
+    }
+    return m_image;
+}
+
 // ------------------------------------------------------------------------
 
 FontFamilyValueImpl::FontFamilyValueImpl( const QString &string)
