@@ -29,7 +29,6 @@
 #include "html/html_miscimpl.h"
 
 #include "xml/dom_docimpl.h"
-#include "misc/htmlhashes.h"
 
 using namespace DOM;
 
@@ -476,16 +475,20 @@ void HTMLInputElement::setReadOnly( bool _readOnly )
 }
 
 /* The next two are provided for backwards compatibility. */
+#ifndef KDE_NO_DEPRECATED
 DOMString HTMLInputElement::size() const
 {
     if(!impl) return DOMString();
     return static_cast<ElementImpl*>(impl)->getAttribute(ATTR_SIZE);
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void HTMLInputElement::setSize( const DOMString &value )
 {
     if(impl) static_cast<ElementImpl*>(impl)->setAttribute(ATTR_SIZE, value);
 }
+#endif
 
 long HTMLInputElement::getSize() const
 {
@@ -501,8 +504,8 @@ void HTMLInputElement::setSize( long value )
 DOMString HTMLInputElement::src() const
 {
     if(!impl) return DOMString();
-    DOMString s = static_cast<ElementImpl*>(impl)->getAttribute(ATTR_SRC);
-    return !s.isNull() ? impl->document()->completeURL( s.string() ) : s;
+    const DOMString s = static_cast<ElementImpl*>(impl)->getAttribute(ATTR_SRC).trimSpaces();
+    return !s.isNull() ? impl->document()->completeURL(s.string()) : s;
 }
 
 void HTMLInputElement::setSrc( const DOMString &value )
@@ -850,7 +853,7 @@ HTMLFormElement HTMLSelectElement::form() const
 HTMLCollection HTMLSelectElement::options() const
 {
     if(!impl) return HTMLCollection();
-    return HTMLCollection(((HTMLSelectElementImpl*)impl)->options()); 
+    return HTMLCollection(((HTMLSelectElementImpl*)impl)->options());
 }
 
 bool HTMLSelectElement::disabled() const
