@@ -36,7 +36,6 @@
 #include "css/cssstyleselector.h"
 #include "css/cssproperties.h"
 #include "css/cssvalues.h"
-#include "css/csshelper.h"
 #include "xml/dom_textimpl.h"
 #include "xml/dom_docimpl.h"
 #include "xml/dom2_eventsimpl.h"
@@ -265,53 +264,49 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
 #endif
 
     QByteArray form_data;
-    form_data.resize(0);
-
-    QByteArray enc_string = ""; // used for non-multipart data
+    QByteArray enc_string; // used for non-multipart data
 
     bool useMultipart = m_multipart && m_post; // as multipart is ignored for GET
 
     // find out the QTextcodec to use
     QString str = m_acceptcharset.string();
-    const QChar space(' ');
-    const unsigned int strLength = str.length();
-    for(unsigned int i=0; i < strLength; ++i) if(str[i].toLatin1() == ',') str[i] = space;
-    const QStringList charsets = str.split(' ');
     QTextCodec* codec = 0;
-    KHTMLView *view = document()->view();
-    {
-        QStringList::ConstIterator it = charsets.begin();
-        const QStringList::ConstIterator itEnd = charsets.end();
+    bool codecOk = false;
 
-        for ( ; it != itEnd; ++it )
-        {
-            QString enc = (*it);
-            if(enc.contains("UNKNOWN"))
-            {
-                // use standard document encoding
-                enc = "ISO 8859-1";
-                if(view && view->part())
-                    enc = view->part()->encoding();
-            }
-            if((codec = KGlobal::charsets()->codecForName(enc.toLatin1().constData())))
-                break;
+    if (str == "UNKNOWN") {
+        // no accept-charset attribute, try document encoding
+        KHTMLView *view = document()->view();
+        if (view && view->part()) {
+            codec = KGlobal::charsets()->codecForName(view->part()->encoding(), codecOk);
         }
     }
-    if(!codec)
+
+    if (!codecOk) {
+        str.replace(QLatin1Char(','), QLatin1Char(' '));
+        const QStringList charsets = str.split(QLatin1Char(' '), QString::SkipEmptyParts);
+        for (QStringList::ConstIterator it = charsets.constBegin(); it != charsets.constEnd(); ++it) {
+            codec = KGlobal::charsets()->codecForName((*it), codecOk);
+            if (codecOk) {
+                break;
+            }
+        }
+    }
+
+    if (!codecOk) {
+        // none of requested could be used fallback to UTF-8
+        codec = KGlobal::charsets()->codecForName("UTF-8", codecOk);
+    }
+
+    if (!codecOk)
         codec = QTextCodec::codecForLocale();
 
     // we need to map visual hebrew to logical hebrew, as the web
     // server alsways expects responses in logical ordering
     if ( codec->mibEnum() == 11 )
-	codec = QTextCodec::codecForMib( 85 );
-
-    m_encCharset = codec->name();
-    const unsigned int m_encCharsetLength = m_encCharset.length();
-    for(unsigned int i=0; i < m_encCharsetLength; ++i)
-        m_encCharset[i] = m_encCharset[i].toLatin1() == ' ' ? QChar('-') : m_encCharset[i].toLower();
+        codec = QTextCodec::codecForMib( 85 );
 
     QStringList fileUploads, fileNotUploads;
-    
+
     /**
      Frameworks such as mootools Sortables expect form element values to be submitted in 
      tree order (and HTML5 specifies this behavior); however formElements need not be 
@@ -450,31 +445,29 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
     if (useMultipart)
         enc_string = QString("--" + m_boundary + "--\r\n").toLatin1().constData();
 
-    const int old_size = form_data.size();
-    form_data.resize( form_data.size() + enc_string.length() );
-    memcpy(form_data.data() + old_size, enc_string.data(), enc_string.length() );
-
+    form_data.append(enc_string.constData());
     ok = true;
     return form_data;
 }
 
-void HTMLFormElementImpl::setEnctype( const DOMString& type )
+void HTMLFormElementImpl::setEnctype(const DOMString& type)
 {
-    if(type.string().indexOf("multipart", 0, Qt::CaseInsensitive) != -1 || type.string().indexOf("form-data", 0, Qt::CaseInsensitive) != -1)
-    {
+    if (type.isEmpty()) { // optimization: majority of real world cases
+        m_enctype = "application/x-www-form-urlencoded";
+        m_multipart = false;
+    }
+    else if (strcasecmp(type, "multipart/form-data") == 0) {
         m_enctype = "multipart/form-data";
         m_multipart = true;
-    } else if (type.string().indexOf("text", 0, Qt::CaseInsensitive) != -1 || type.string().indexOf("plain", 0, Qt::CaseInsensitive) != -1)
-    {
+    }
+    else if (strcasecmp(type, "text/plain") == 0) {
         m_enctype = "text/plain";
         m_multipart = false;
     }
-    else
-    {
+    else {
         m_enctype = "application/x-www-form-urlencoded";
         m_multipart = false;
     }
-    m_encCharset.clear();
 }
 
 static QString calculateAutoFillKey(const HTMLFormElementImpl& e)
@@ -686,19 +679,18 @@ void HTMLFormElementImpl::submit(  )
         }
 #endif // KHTML_NO_WALLET
 
-        DOMString value = getAttribute(ATTR_ACTION);
-        DOMString url = khtml::parseURL(value);
+        QString url = getAttribute(ATTR_ACTION).trimSpaces().string();
         // ignore base url if 'action' attribute is empty.
-        if (value.isEmpty())
+        if (url.isEmpty())
             url = formUrl.url();
         if(m_post) {
-            view->part()->submitForm( "post", url.string(), form_data,
+            view->part()->submitForm( "post", url, form_data,
                                       m_target.string(),
                                       enctype().string(),
                                       m_boundary );
         }
         else {
-            view->part()->submitForm( "get", url.string(), form_data,
+            view->part()->submitForm( "get", url, form_data,
                                       m_target.string() );
         }
     }
@@ -2455,11 +2447,11 @@ void HTMLSelectElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_SIZE:
-        m_size = qMax( attr->val()->toInt(), 1 );
+        m_size = qMax(attr->value().toInt(), 1);
         setChanged();
         break;
     case ATTR_WIDTH:
-        m_minwidth = qMax( attr->val()->toInt(), 0 );
+        m_minwidth = qMax(attr->value().toInt(), 0);
         break;
     case ATTR_MULTIPLE:
         m_multiple = (attr->val() != 0);
@@ -2526,10 +2518,7 @@ bool HTMLSelectElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
         (items[0]->id() == ID_OPTION && !items[0]->disabled()) ) {
         HTMLOptionElementImpl* const option = static_cast<HTMLOptionElementImpl*>(items[0]);
         encoded_values += enc_name;
-        if (option->value().isNull())
-            encoded_values += fixUpfromUnicode(codec, option->text().string());
-        else
-            encoded_values += fixUpfromUnicode(codec, option->value().string());
+        encoded_values += fixUpfromUnicode(codec, option->value().string());
         successful = true;
     }
 
@@ -2809,7 +2798,7 @@ DOMString HTMLOptionElementImpl::value() const
     if ( !m_value.isNull() )
         return m_value;
     // Use the text if the value wasn't set.
-    return text().string();
+    return text();
 }
 
 void HTMLOptionElementImpl::setValue(DOMStringImpl* value)
