@@ -24,7 +24,6 @@
 #include "html/html_formimpl.h"
 #include "html/html_documentimpl.h"
 
-#include "misc/htmlhashes.h"
 #include "khtmlview.h"
 #include "khtml_part.h"
 
@@ -37,7 +36,6 @@
 #include "css/cssstyleselector.h"
 #include "css/cssproperties.h"
 #include "css/cssvalues.h"
-#include "css/csshelper.h"
 #include "xml/dom2_eventsimpl.h"
 
 #include <QtCore/QCharRef>
@@ -86,10 +84,9 @@ void HTMLImageElementImpl::parseAttribute(AttributeImpl *attr)
         setChanged();
 
         //Start loading the image already, to generate events
-        DOMString url = attr->value();
-        if (!url.isEmpty()) { //### why do we not hide or something when setting this?
-            DOMString parsedURL = khtml::parseURL(url);
-            CachedImage* newImage = document()->docLoader()->requestImage(parsedURL);
+        const DOMString imgSrcUrl = attr->value().trimSpaces();
+        if (!imgSrcUrl.isEmpty()) { //### why do we not hide or something when setting this?
+            CachedImage* newImage = document()->docLoader()->requestImage(imgSrcUrl);
             if (newImage && newImage != m_image) {
                 CachedImage* oldImage = m_image;
                 loadEventSent = false;
@@ -99,9 +96,12 @@ void HTMLImageElementImpl::parseAttribute(AttributeImpl *attr)
                     oldImage->deref(this);
             }
 
-            KUrl fullURL = document()->completeURL(parsedURL.string());
-            if (document()->URL().host() != fullURL.host() || document()->URL().port() != fullURL.port())
-                unsafe = true;
+            if (m_image) {
+                const KUrl fullURL = KUrl(m_image->url().string());
+                if (document()->origin()->taintsCanvas(fullURL)) {
+                    unsafe = true;
+                }
+            }
         }
     }
     break;
@@ -151,7 +151,7 @@ void HTMLImageElementImpl::parseAttribute(AttributeImpl *attr)
         if ( attr->value()[0] == '#' )
             usemap = attr->value().lower();
         else {
-            QString url = document()->completeURL( khtml::parseURL( attr->value() ).string() );
+            QString url = document()->completeURL(attr->value().trimSpaces().string());
             // ### we remove the part before the anchor and hope
             // the map is on the same html page....
             usemap = url;
@@ -438,7 +438,7 @@ void HTMLMapElementImpl::parseAttribute(AttributeImpl *attr)
         }
         else {
             // add name with full url:
-            QString url = document()->completeURL( khtml::parseURL( attr->value() ).string() );
+            const QString url = document()->completeURL(attr->value().trimSpaces().string());
             if(document()->isHTMLDocument())
                 static_cast<HTMLDocumentImpl*>(document())->mapMap[url] = this;
         }
@@ -572,12 +572,12 @@ QRegion HTMLAreaElementImpl::getRegion(int width_, int height_) const
                             m_coords[(i<<1)+1].minWidth(height_));
         region = QRegion(points);
     }
-    else if (shape==Circle && m_coordsLen>=3 || shape==Unknown && m_coordsLen == 3) {
+    else if ((shape==Circle && m_coordsLen>=3) || (shape==Unknown && m_coordsLen == 3)) {
         int r = qMin(m_coords[2].minWidth(width_), m_coords[2].minWidth(height_));
         region = QRegion(m_coords[0].minWidth(width_)-r,
                          m_coords[1].minWidth(height_)-r, 2*r, 2*r,QRegion::Ellipse);
     }
-    else if (shape==Rect && m_coordsLen>=4 || shape==Unknown && m_coordsLen == 4) {
+    else if ((shape==Rect && m_coordsLen>=4) || (shape==Unknown && m_coordsLen == 4)) {
         int x0 = m_coords[0].minWidth(width_);
         int y0 = m_coords[1].minWidth(height_);
         int x1 = m_coords[2].minWidth(width_);
