@@ -1696,6 +1696,18 @@ void DocumentImpl::finishParsing (  )
 
 QString DocumentImpl::completeURL(const QString& url) const
 {
+    if (url.startsWith(QLatin1Char('#'))) {
+        const QString ref = KUrl::fromPercentEncoding(url.mid(1).toUtf8());
+        KUrl u = baseURL();
+        if (ref.isEmpty()) {
+            u.setHTMLRef("");
+        }
+        else {
+            u.setHTMLRef(ref);
+        }
+        return u.url();
+    }
+
     return KUrl(baseURL(),url /*,m_decoderMibEnum*/).url();
 }
 
@@ -1979,7 +1991,7 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
                 v->part()->scheduleRedirection(delay, v->part()->url().url() );
         } else {
             pos++;
-            while(pos < (int)str.length() && str[pos].isSpace()) pos++;
+            while(pos < str.length() && str[pos].isSpace()) pos++;
             str = str.mid(pos);
             if(str.indexOf("url", 0, Qt::CaseInsensitive ) == 0)  str = str.mid(3);
             str = str.trimmed();
@@ -1987,7 +1999,8 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
             while (str.length() && (str[str.length()-1] == ';' || str[str.length()-1] == ',')) {
                 str.resize(str.length()-1);
             }
-            QString newURL = document()->completeURL(str.trimmed());
+            str = DOMString(str).trimSpaces().string();
+            QString newURL = document()->completeURL(str);
             if ( ok )
                 v->part()->scheduleRedirection(delay, newURL,  delay < 2 || newURL == URL().url());
         }
@@ -2047,7 +2060,7 @@ bool DocumentImpl::prepareMouseEvent( bool readonly, int _x, int _y, MouseEvent
             //qDebug("urlnode: %s  (%d)", getTagName(renderInfo.URLElement()->id()).string().toLatin1().constData(), renderInfo.URLElement()->id());
 
             ElementImpl* e =  static_cast<ElementImpl*>(renderInfo.URLElement());
-            const DOMString href = e->getAttribute(ATTR_HREF).string().trimmed();
+            const DOMString href = e->getAttribute(ATTR_HREF).trimSpaces();
             const DOMString target = e->getAttribute(ATTR_TARGET);
 
             if (!target.isNull() && !href.isNull()) {
