@@ -110,18 +110,18 @@ void FileProtocol::copy( const KUrl &srcUrl, const KUrl &destUrl,
     bool dest_exists = ( KDE_lstat( _dest.data(), &buff_dest ) != -1 );
     if ( dest_exists )
     {
+        if ( same_inode( buff_dest, buff_src) )
+        {
+            error(KIO::ERR_IDENTICAL_FILES, dest);
+            return;
+        }
+
         if (S_ISDIR(buff_dest.st_mode))
         {
            error(KIO::ERR_DIR_ALREADY_EXIST, dest);
            return;
         }
 
-	if ( same_inode( buff_dest, buff_src) )
-	{
-	    error(KIO::ERR_IDENTICAL_FILES, dest);
-	    return;
-	}
-
         if (!(_flags & KIO::Overwrite))
         {
            error(KIO::ERR_FILE_ALREADY_EXIST, dest);
@@ -382,8 +382,8 @@ void FileProtocol::listDir( const KUrl& url)
             entry.insert(KIO::UDSEntry::UDS_NAME, filename);
 #ifdef HAVE_DIRENT_D_TYPE
             entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
-                         (ep->d_type & DT_DIR) ? S_IFDIR : S_IFREG );
-            const bool isSymLink = (ep->d_type & DT_LNK);
+                         (ep->d_type == DT_DIR) ? S_IFDIR : S_IFREG );
+            const bool isSymLink = (ep->d_type == DT_LNK);
 #else
             // oops, no fast way, we need to stat (e.g. on Solaris)
             if (KDE_lstat(ep->d_name, &st) == -1) {
@@ -439,18 +439,18 @@ void FileProtocol::rename( const KUrl &srcUrl, const KUrl &destUrl,
     bool dest_exists = ( KDE_lstat( _dest.data(), &buff_dest ) != -1 );
     if ( dest_exists )
     {
+        if ( same_inode( buff_dest, buff_src) )
+        {
+            error(KIO::ERR_IDENTICAL_FILES, dest);
+            return;
+        }
+
         if (S_ISDIR(buff_dest.st_mode))
         {
            error(KIO::ERR_DIR_ALREADY_EXIST, dest);
            return;
         }
 
-	if ( same_inode( buff_dest, buff_src) )
-	{
-	    error(KIO::ERR_IDENTICAL_FILES, dest);
-	    return;
-	}
-
         if (!(_flags & KIO::Overwrite))
         {
            error(KIO::ERR_FILE_ALREADY_EXIST, dest);
